from multisig_ci.safes import safe
from multisig_ci.sentry_wrapper import custom_sentry_trace, CustomSentryTransaction, CustomSentrySpan

@custom_sentry_trace
def _tenderly_fork(safe, timeout_seconds):
   import requests
   import brownie
   from brownie import chain

   fork_base_url = "https://simulate.yexporter.io/fork"
   payload = {"network_id": str(chain.id)}
   resp = requests.post(fork_base_url, headers={}, json=payload)
   fork_id = resp.json()["simulation_fork"]["id"]
   fork_rpc_url = f"https://rpc.tenderly.co/fork/{fork_id}"
   print(fork_rpc_url)
   tenderly_provider = brownie.web3.HTTPProvider(fork_rpc_url, {"timeout": timeout_seconds})
   brownie.web3.provider = tenderly_provider
   safe.w3.provider = tenderly_provider
   print(f"https://dashboard.tenderly.co/yearn/yearn-web/fork/{fork_id}")

@custom_sentry_trace
def sign(nonce_arg = None, skip_preview = False, post_tx = False, tenderly_fork = False, tenderly_timeout_seconds = 60):
    global SENTRY_IMPORTED
    def _sign(func):
        def wrapper():
            if safe.is_ci and safe.is_send:
                op = "ci_send"
            else:
                op = "ci_dry_run"

            with CustomSentryTransaction(op=op, name=func.__name__):
                if tenderly_fork:
                    with CustomSentrySpan(description="tenderly_fork"):
                        _tenderly_fork(safe, tenderly_timeout_seconds)

                with CustomSentrySpan(description="run_func"):
                    func()
                
                with CustomSentrySpan(description="build_tx"):
                    safe_tx = safe.multisend_from_receipts(safe_nonce=nonce)

                if not skip_preview:
                    with CustomSentrySpan(description="preview_tx"):
                        safe.preview(safe_tx, call_trace=False)

                if not post_tx and not safe.is_ci:
                    print("dry-run finished, run again with @sign(post_tx = True) to sign and submit the tx.")
                else:
                    if safe.is_ci and not safe.is_send:
                        print("CI dry-run enabled, set send to true to run to completion")
                        return

                    with CustomSentrySpan(description="sign_tx"):
                        safe.sign_transaction(safe_tx)
                    with CustomSentrySpan(description="post_tx"):
                        safe.post_transaction(safe_tx)

        return wrapper

    if callable(nonce_arg):
        nonce = None
        return _sign(nonce_arg)

    nonce = int(nonce_arg) if nonce_arg else nonce_arg
    return _sign



from subprocess import Popen
from tenacity import *
import sys, time, os, signal, psutil
from multisig_ci.sentry_wrapper import custom_sentry_trace, CustomSentryTransaction

home_directory = os.environ.get("HOME")
signal_file_path = os.path.join(home_directory, "alive.signal")
nonce_file_path = os.path.join(home_directory, "nonce.txt")
current_try_count = 0


@custom_sentry_trace
@retry(stop=stop_after_attempt(5))
@custom_sentry_trace
def run_brownie(args):
    global current_try_count

    # Kill processes to make sure we start clean
    kill_process_by_cmdline("ganache-cli")
    kill_process_by_name("brownie")
    kill_process_by_cmdline("anvil")

    if os.path.exists(signal_file_path) and current_try_count == 0:
        os.remove(signal_file_path)
        print("cleaning up signal from last run")

    if os.path.exists(nonce_file_path):
        if current_try_count == 0:
            os.remove(nonce_file_path)
        else:
            print("nonce found, aborting before we trigger another tx")
            exit(1)

    p = Popen(args)

    # sleep 10, 20, 30, 40, 50, or 60 seconds based on retries
    sleep_time = 10 + min(current_try_count * 10, 50)
    print(f"waiting for alive signal, sleeping for {sleep_time} seconds")
    time.sleep(sleep_time)

    current_try_count += 1

    if not os.path.exists(signal_file_path):
        print(f"alive signal not found, killing brownie, ganache, and anvil. queuing try #{current_try_count}")
        p.terminate()
        kill_process_by_cmdline("ganache-cli")
        kill_process_by_cmdline("anvil")
        raise Exception()

    print("found alive signal, waiting for process to complete")
    exit_code = p.wait()
    os.remove(signal_file_path)
    exit(exit_code)

@custom_sentry_trace
def kill_process_by_cmdline(cmdline_arg_find):
    for proc in psutil.process_iter():
        for cmdline_arg in proc.cmdline():
            if cmdline_arg_find in cmdline_arg:
                pid = proc.pid
                os.kill(int(pid), signal.SIGKILL)

@custom_sentry_trace
def kill_process_by_name(proc_name):
    for proc in psutil.process_iter():
        if proc_name == proc.name():
            pid = proc.pid
            os.kill(int(pid), signal.SIGKILL)


if __name__ == "__main__":
    with CustomSentryTransaction(op="run_brownie", name="RunBrownie"):
        run_brownie(sys.argv[1:])



import requests
import sys
import os
import re

SEND_MESSAGE_FORMAT = "https://api.telegram.org/bot{0}/sendmessage"
PIN_MESSAGE_FORMAT = "https://api.telegram.org/bot{0}/pinchatmessage?chat_id={1}&message_id={2}&disable_web_page_preview=true"

escape_chars = r'*'

def send_message(bot_token, chat_id, chat_message):
    print(chat_message)
    chat_message = re.sub(f'([{re.escape(escape_chars)}])', r'\\\1', chat_message)
    print(chat_message)
    uri = SEND_MESSAGE_FORMAT.format(bot_token)
    res = requests.post(uri, data={'chat_id':chat_id, 'text': chat_message, 'disable_web_page_preview': True, 'parse_mode': 'markdown'})
    print(res)
    print(res.text)
    if res.ok:
        return res.json(), res.ok
    else:
        return None, res.ok

def pin_message(bot_token, chat_id, chat_message_id):
    uri = PIN_MESSAGE_FORMAT.format(bot_token, chat_id, chat_message_id)
    res = requests.get(uri)
    print(res)
    print(res.text)
    if res.ok:
        return res.json(), res.ok
    else:
        return None, res.ok

def send_and_pin_message(bot_token, chat_id, chat_message):
    retryCount = 0
    ok = False
    data = None
    while retryCount < 3 and not ok:
        data, ok = send_message(bot_token, chat_id, chat_message)
        retryCount += 1
    
    if not ok:
        raise Exception("failed to post message")
    
    retryCount = 0
    print(data)
    message_id = data['result']['message_id']
    data = None
    ok = False
    while retryCount < 3 and not ok:
        data, ok = pin_message(bot_token, chat_id, message_id)
        retryCount += 1
    
    if not ok:
        raise Exception("failed to pin")

if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1] == 'send_and_pin_message':
        try:
            send_and_pin_message(sys.argv[2], sys.argv[3], os.getenv("TELEGRAM_MESSAGE"))
        except Exception as e:
            print(e)
            exit(1)


import requests
import os, stat
import multisig_ci.ci_override

vyper_releases = [
    "https://github.com/vyperlang/vyper/releases/download/v0.3.3/vyper.0.3.3+commit.48e326f0.linux"
    "https://github.com/vyperlang/vyper/releases/download/v0.3.2/vyper.0.3.2+commit.3b6a4117.linux"
    "https://github.com/vyperlang/vyper/releases/download/v0.3.1/vyper.0.3.1.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.3.0/vyper.0.3.0+commit.8a23feb.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.16/vyper.0.2.16+commit.59e1bdd.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.15/vyper.0.2.15+commit.6e7dba7.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.12/vyper.0.2.12+commit.2c6842c.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.11/vyper.0.2.11+commit.5db35ef.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.8/vyper.0.2.8+commit.069936f.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.7/vyper.0.2.7+commit.0b3f3b3.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.6/vyper.0.2.6+commit.35467d5.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.5/vyper.0.2.5+commit.a0c561c.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.4/vyper.0.2.4+commit.7949850.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.3/vyper.0.2.3+commit.006968f.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.2/vyper.0.2.2+commit.337c2ef.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.1/vyper.0.2.1+commit.cac3d7d.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.2.0/vyper.0.2.0+commit.d2c0c87.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.1.0-beta.17/vyper.0.1.0-beta.17+commit.0671b7b.linux",
    "https://github.com/vyperlang/vyper/releases/download/v0.1.0-beta.16/vyper.0.1.0-beta.16+commit.5e4a94a.linux",
]

solc_url_prefix = "https://solc-bin.ethereum.org/linux-amd64/solc-linux-amd64-"

solc_release_versions = [
    "v0.8.13+commit.abaa5c0e"
    "v0.8.12+commit.f00d7308"
    "v0.8.11+commit.d7f03943",
    "v0.8.10+commit.fc410830",
    "v0.8.9+commit.e5eed63a",
    "v0.8.8+commit.dddeac2f",
    "v0.8.7+commit.e28d00a7",
    "v0.8.6+commit.11564f7e",
    "v0.8.5+commit.a4f2e591",
    "v0.8.4+commit.c7e474f2",
    "v0.8.3+commit.8d00100c",
    "v0.8.2+commit.661d1103",
    "v0.8.1+commit.df193b15",
    "v0.8.0+commit.c7dfd78e",
    "v0.7.6+commit.7338295f",
    "v0.7.5+commit.eb77ed08",
    "v0.7.4+commit.3f05b770",
    "v0.7.3+commit.9bfce1f6",
    "v0.7.2+commit.51b20bc0",
    "v0.7.1+commit.f4a555be",
    "v0.7.0+commit.9e61f92b",
    "v0.6.12+commit.27d51765",
    "v0.6.11+commit.5ef660b1",
    "v0.6.10+commit.00c0fcaf",
    "v0.6.9+commit.3e3065ac",
    "v0.6.8+commit.0bbfe453",
    "v0.6.7+commit.b8d736ae",
    "v0.6.6+commit.6c089d02",
    "v0.6.5+commit.f956cc89",
    "v0.6.4+commit.1dca32f3",
    "v0.6.3+commit.8dda9521",
    "v0.6.2+commit.bacdbe57",
    "v0.6.1+commit.e6f7d5a4",
    "v0.6.0+commit.26b70077",
    "v0.5.17+commit.d19bba13",
    "v0.5.16+commit.9c3226ce",
    "v0.5.15+commit.6a57276f",
    "v0.5.14+commit.01f1aaa4",
    "v0.5.13+commit.5b0b510c",
    "v0.5.12+commit.7709ece9",
    "v0.5.11+commit.22be8592",
    "v0.5.11+commit.c082d0b4",
    "v0.5.10+commit.5a6ea5b1",
    "v0.5.9+commit.c68bc34e",
    "v0.5.9+commit.e560f70d",
    "v0.5.8+commit.23d335f2",
    "v0.5.7+commit.6da8b019",
    "v0.5.6+commit.b259423e",
    "v0.5.5+commit.47a71e8f",
    "v0.5.4+commit.9549d8ff",
    "v0.5.3+commit.10d17f24",
    "v0.5.2+commit.1df8f40c",
    "v0.5.1+commit.c8a2cb62",
    "v0.5.0+commit.1d4f565a",
    "v0.4.26+commit.4563c3fc",
    "v0.4.25+commit.59dbf8f1",
    "v0.4.24+commit.e67f0147",
    "v0.4.23+commit.124ca40d",
    "v0.4.22+commit.4cb486ee",
    "v0.4.21+commit.dfe3193c",
    "v0.4.20+commit.3155dd80",
    "v0.4.19+commit.c4cbbb05",
    "v0.4.18+commit.9cf6e910",
    "v0.4.17+commit.bdeb9e52",
    "v0.4.16+commit.d7661dd9",
    "v0.4.15+commit.8b45bddb",
    "v0.4.15+commit.bbb8e64f",
    "v0.4.14+commit.c2215d46",
    "v0.4.13+commit.0fb4cb1a",
    "v0.4.12+commit.194ff033",
    "v0.4.11+commit.68ef5810",
]

home_directory = os.environ.get("HOME")

def hydrate_compiler_cache():
    for vyper_release in vyper_releases:
        mod = vyper_release.index("+") if "+" in vyper_release else vyper_release.index(".linux")
        name = vyper_release[vyper_release.index("vyper.") : mod]
        print("Downloading " + name)
        r = requests.get(vyper_release, allow_redirects=True)
        vvm_folder = os.path.join(home_directory, ".vvm/")
        if not os.path.exists(vvm_folder):
            os.mkdir(vvm_folder)
        file_name = vvm_folder + name.replace(".", "-", 1)
        with open(file_name, "wb+") as f:
            f.write(r.content)
            st = os.stat(file_name)
            os.chmod(file_name, st.st_mode | stat.S_IEXEC)

    for solc_release_version in solc_release_versions:
        solc_release_url = solc_url_prefix + solc_release_version
        prefix = "solc-linux-amd64"
        start = solc_release_url.index(prefix) + len(prefix)
        end = solc_release_url.index("+")
        name = "solc" + solc_release_url[start:end]
        print("Downloading " + name)
        r = requests.get(solc_release_url, allow_redirects=True)
        solcx_folder = os.path.join(home_directory, ".solcx/")
        if not os.path.exists(solcx_folder):
            os.mkdir(solcx_folder)
        file_name = os.path.join(home_directory, ".solcx/") + name
        with open(file_name, "wb+") as f:
            f.write(r.content)
            st = os.stat(file_name)
            os.chmod(file_name, st.st_mode | stat.S_IEXEC)


import os

SENTRY_IMPORTED=False
try:
    import sentry_sdk
    import sentry_sdk.integrations.excepthook
    sentry_dsn = os.environ.get("SENTRY_DSN")
    debug_from_env = os.environ.get("SENTRY_DEBUG", False)
    sentry_sdk.init(
        dsn=sentry_dsn,
        sample_rate=1.0,
        traces_sample_rate=1.0,
        profiles_sample_rate=1.0,
        integrations=[sentry_sdk.integrations.excepthook.ExcepthookIntegration(always_run=True)],
        debug=debug_from_env,
    )
    SENTRY_IMPORTED = True
except Exception as e:
    print(e)
    pass

def custom_sentry_trace(func=None):
    global SENTRY_IMPORTED
    if SENTRY_IMPORTED:
        return sentry_sdk.trace(func)
    return func

class CustomSentryTransaction:
    def __init__(self, op, name):
        self.op = op
        self.name = name
        self.tx = None

    def __enter__(self):
        global SENTRY_IMPORTED
        if SENTRY_IMPORTED:
            self.tx = sentry_sdk.start_transaction(op=self.op, name=self.name)
            self.tx.__enter__()

        return self.tx

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.tx is not None:
            self.tx.__exit__(exc_type, exc_val, exc_tb)

class CustomSentrySpan:
    def __init__(self, description):
        self.description = description
        self.span = None

    def __enter__(self):
        global SENTRY_IMPORTED
        if SENTRY_IMPORTED:
            self.span = sentry_sdk.start_span(description=self.description)
            self.span.__enter__()

        return self.span

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.span is not None:
            self.span.__exit__(exc_type, exc_val, exc_tb)



from multisig_ci.run_brownie import run_brownie
from multisig_ci.hydrate_ci_cache import hydrate_compiler_cache
import multisig_ci.telegram as telegram
import sys
import os

if __name__ == '__main__':
    if 'hydrate_compiler_cache' in sys.argv:
        hydrate_compiler_cache()
        sys.exit(0)
    
    if len(sys.argv) > 1 and sys.argv[1] == 'send_and_pin_message':
        try:
            telegram.send_and_pin_message(sys.argv[2], sys.argv[3], os.getenv("TELEGRAM_MESSAGE"))
            exit(0)
        except:
            exit(1)

    run_brownie(sys.argv[1:])


import os
from copy import copy
from brownie_safe import BrownieSafeBase, BrownieSafe
from brownie_safe import ExecutionFailure, PATCHED_SAFE_VERSIONS
from brownie import accounts, network, chain, Contract
from gnosis.safe.safe_tx import SafeTx
from gnosis.safe.safe import SafeV111, SafeV120, SafeV130, SafeV141
from gnosis.eth.ethereum_client import EthereumNetwork
from eth_abi import encode
from eth_utils import keccak
from typing import Optional, Union
from brownie.network.account import LocalAccount, Account
from brownie.network.contract import _explorer_tokens
from brownie._config import CONFIG
from _pytest.monkeypatch import MonkeyPatch
from brownie.network.rpc import anvil 
from multisig_ci.sentry_wrapper import custom_sentry_trace


@custom_sentry_trace
def mine_override(timestamp: Optional[int] = None) -> None:
    if timestamp:
        anvil._request("evm_setNextBlockTimestamp", [timestamp])
    anvil._request("evm_mine", [])

monkeypatch = MonkeyPatch()
monkeypatch.setattr(anvil, 'mine', mine_override)

# CI horribleness lurks below
# If running in CI, let's override BrownieSafeBase.post_transaction so
# that it writes a file with the nonce. This is used to later tag
# the pull request with a label matching the nonce

DELEGATE_ADDRESS = os.environ.get("DELEGATE_ADDRESS")
home_directory = os.environ.get("HOME")
BASE_CHAIN_ID = 8453
FANTOM_CHAIN_ID = 250

gnosis_frontend_urls = {
    1: 'https://app.safe.global/transactions/queue?safe=eth:{0}',
    4: 'https://app.safe.global/transactions/queue?safe=rin:{0}',
    5: 'https://app.safe.global/transactions/queue?safe=gor:{0}',
    10: 'https://app.safe.global/transactions/queue?safe=oeth:{0}',
    56: 'https://app.safe.global/transactions/queue?safe=bnb:{0}',
    100: 'https://app.safe.global/transactions/queue?safe=xdai:{0}',
    137: 'https://app.safe.global/transactions/queue?safe=matic:{0}',
    250: 'https://safe.fantom.network/transactions/queue?safe=ftm:{0}',
    8453: 'https://app.safe.global/transactions/queue?safe=base:{0}',
    42161: 'https://app.safe.global/transactions/queue?safe=arb1:{0}',
}

_explorer_tokens['basescan'] = 'BASESCAN_TOKEN'

class DelegateSafeBase(BrownieSafeBase):
    @custom_sentry_trace
    def __init__(self, address, ethereum_client):
        if CONFIG.network_type != "development":
            acct = Account(address)
            if acct not in accounts._accounts:
                accounts._accounts.append(acct)

        super().__init__(address, ethereum_client)

    @property
    def is_ci(self):
        return os.environ.get("CI", "").lower() == "true"

    @property
    def is_send(self):
        return os.environ.get("GITHUB_ACTION_SEND", "").lower() == "true"

    @custom_sentry_trace
    def post_transaction(self, safe_tx: SafeTx):
        super().post_transaction(safe_tx)

        if self.is_ci and self.is_send:
            if "{1}" in self.frontend_url:
                formatted_frontend_url = self.frontend_url.format(self.address, safe_tx.safe_tx_hash.hex())
            else:
                formatted_frontend_url = self.frontend_url.format(self.address)

            with open(os.getenv('GITHUB_ENV'), 'a') as f:
                f.write("SAFE_LINK={0}\n".format(formatted_frontend_url))
            with open(os.path.join(home_directory, "safe.txt"), "w") as f:
                f.write(str(formatted_frontend_url))
            with open(os.getenv('GITHUB_ENV'), 'a') as f:
                f.write("NONCE={0}\n".format(str(safe_tx.safe_nonce)))
            with open(os.path.join(home_directory, "nonce.txt"), "w") as f:
                f.write(str(safe_tx.safe_nonce))
            exit(0)

    @custom_sentry_trace
    def preview_tx(self, safe_tx: SafeTx, events=True, call_trace=False):
        if self.is_ci:
            events = False
            call_trace = False

        tx = copy(safe_tx)
        safe = Contract.from_abi('Gnosis Safe', self.address, self.contract.abi)
        # Replace pending nonce with the subsequent nonce, this could change the safe_tx_hash
        tx.safe_nonce = safe.nonce()
        # Forge signatures from the needed amount of owners, skip the one which submits the tx
        # Owners must be sorted numerically, sorting as checksum addresses may yield wrong order
        threshold = safe.getThreshold()
        sorted_owners = sorted(safe.getOwners(), key=lambda x: int(x, 16))
        owners = [accounts.at(owner, force=True) for owner in sorted_owners[:threshold]]
        # Signautres are encoded as [bytes32 r, bytes32 s, bytes8 v]
        # Pre-validated signatures are encoded as r=owner, s unused and v=1.
        # https://docs.gnosis.io/safe/docs/contracts_signatures/#pre-validated-signatures
        tx.signatures = b''.join([encode(['address', 'uint'], [str(owner), 0]) + b'\x01' for owner in owners])

        # approvedHashes are in slot 8 and have type of mapping(address => mapping(bytes32 => uint256))
        for owner in owners[:threshold]:
            outer_key = keccak(encode(['address', 'uint'], [str(owner), 8]))
            slot = int.from_bytes(keccak(tx.safe_tx_hash + outer_key), 'big')
            self.set_storage(tx.safe_address, slot, 1)

        payload = tx.w3_tx.build_transaction()
        receipt = owners[0].transfer(payload['to'], payload['value'], gas_limit=payload['gas'], data=payload['data'])

        if 'ExecutionSuccess' not in receipt.events:
            receipt.info()
            receipt.call_trace(True)
            raise ExecutionFailure()

        if events:
            receipt.info()
        if call_trace:
            receipt.call_trace(True)
        return receipt

    @custom_sentry_trace
    def get_signer(self, signer: Optional[Union[LocalAccount, str]] = None) -> LocalAccount:
        if not self.is_ci:
            return super().get_signer(signer)

        if self.is_send:
            key = os.environ.get("PRIVATE_KEY")
            assert (
                key is not None
            ), "CI environment missing PRIVATE_KEY environment variable. Please add it as a repository secret."
            user = accounts.add(key)
            assert (
                user.address == DELEGATE_ADDRESS
            ), "Delegate address mismatch. Check you have correct private key."
            return user
        
        return None

    @custom_sentry_trace
    def sign_transaction(self, safe_tx: SafeTx, signer=None) -> SafeTx:
        if not self.is_ci:
            return super().sign_transaction(safe_tx, signer)

        if not self.is_send:
            print("CI dry-run enabled, set send to true to run to completion")
            exit(0)
        
        return super().sign_transaction(safe_tx, signer)

class DelegateSafeV111(DelegateSafeBase, SafeV111):
    pass

class DelegateSafeV120(DelegateSafeBase, SafeV120):
    pass

class DelegateSafeV130(DelegateSafeBase, SafeV130):
    pass

class DelegateSafeV141(DelegateSafeBase, SafeV141):
    pass

PATCHED_SAFE_VERSIONS['1.1.1'] = DelegateSafeV111
PATCHED_SAFE_VERSIONS['1.2.0'] = DelegateSafeV120
PATCHED_SAFE_VERSIONS['1.3.0'] = DelegateSafeV130
PATCHED_SAFE_VERSIONS['1.4.1'] = DelegateSafeV141

def DelegateSafe(address, base_url=None, multisend=None):
    """
    Create an BrownieSafeBase from an address or a ENS name and use a default connection.
    """
    if network.chain.id in gnosis_frontend_urls:
        frontend_url = gnosis_frontend_urls[network.chain.id]
    else:
        frontend_url = gnosis_frontend_urls[1]

    if not base_url and network.chain.id == FANTOM_CHAIN_ID:
        base_url = "https://safe-txservice.fantom.network"

    safe = BrownieSafe(address, base_url, multisend)
    safe.frontend_url = frontend_url
    return safe


if os.environ.get("CI", "").lower() == "true":
    with open(os.path.join(home_directory, "alive.signal"), "w") as f:
        f.write("I am alive")
